|**     runtime startoff
|
|	calls main
|	call exit



|**     Calling the Operating system
|
|       Since each 8086 system tends to be different, all operating
|       system features are accessed by calling certain 'magic offsets'
|       in the low code-space of the user program.  There are currently
|       three of these entry points; each is called by a long-call
|       instruction.
|
|       These three magic entry points are defined symbolicly here,
|       for convenient use by library routines.  However, the code which
|       impliments them is stuffed into low code space by the kernal
|       at 'exec' time.  Thus, the offsets of these entry points must
|       not be changed; as they are specified in routine ???? in the
|       kernal.


.globl	_exit, _environ, __syscal
.globl	start
.globl	_main

	.comm   _environ,2.

exit = 1.

start:                          |******   Start executing here

|       These first 128 bytes will be set by the kernel when the
|       module is first executed.  The kernel assumes that the first
|       128 bytes of code space are available to it.  User code
|       must do long calls into one of 8 magic offsets in this
|       128-byte area:

	j       start0
__syscal:
	j       .               | system call           +2
	j       .               | stack growth          +4
	j       .               | stack adjust          +6
	j       .               | reserved              +8
	j       .               | reserved              +10
	j       .               | reserved              +12
	j       .               | reserved              +14
	j       .               | reserved              +16


	.=start+128             | kernal loads first 128 bytes


start0: test	ax,#/FFFF	| I000 check for stack size
	jz	s0		|
	mov	nd,ax		| I000 nd <-- break address
s0:
	mov     bx,sp           | bx->ac,avs,envs
	sub     sp,*4           | space for real arguments
	add     bx,*2
	push    bx              | main (argv,
	push    *-2(bx)         | main (argc, argv
s1:
	add     bx,*2           | next argv
	test    (bx),#/ffff     | look at pointer
	jnz     s1              | loop until zero argv

	add     bx,*2           | first env
	mov     si,sp
	mov     *4(si),bx       | main (argc, argv, envv)
	mov     _environ,bx     | place for people to look for them

	mov     ax,cs           | temporary until new linker, see
	mov     Isgadr+2,ax     |       'Isgadr'...
	mov     Iscadr+2,ax

	calli   _main
	add     sp,*6
	push    ax              | pass return value to exit
s2:     calli   _exit           | call portable exit
	j       s2              | _exit never returns{

	.data
	.word   0       | loc 0 for I/D; null ptr points here.


|       Until we're using the new loader, its a pain generating segmented
|       addresses, and the library routines will call the Ios_?? entrys
|       indirectly through these addresses.  The segment values are filled
|       in when crt0 gets control

	.globl  Isgadr,Iscadr

Isgadr: .word   4               | stack growth
	.word   0
Iscadr: .word   2               | system call
	.word   0


| C library -- signal
|
| signal(n, 0);		/* default action on signal(n) */
| signal(n, 1);		/* ignore signal(n)	M000 */
| signal(n, label);	/* call label(n) on signal(n) */
|
| returns old label, only one level.
|
|	arw	 9 Jan 1982	M000
|		- Modified to make the ignore signal argument as
|		  being exactly one.
|	arw	29 Sep 1982	M001
|		- Mods. to middle model version to correct bugs.

	.globl	_signal
	.comm	_errno,2
	.sigs = 48.

	.text
_signal:
	mov     bx,sp
	mov     cx,*8(bx)       | (cx) = vector base
	mov	dx,*6(bx)	| (dx) = vector offset
	mov     bx,*4(bx)       | (bx) = signal number
	cmp     bx,*16.         | max signal
	ja      sig2            | toobig
	mov     ax,bx           | (ax) = signal #, PRESERVE until syscal
	add     bx,bx           | (bx) = 2*sig
	add	bx,bx		| (bx) = 4*sig
	push	2+sigtab(bx)	| save old base value for return
	push    sigtab(bx)      | save old offset value for return
	mov	2+sigtab(bx),cx	| store users routine base addr in table
	mov     sigtab(bx),dx   | store users routine offset addr in table
	and     dx,dx           | zero argument offset
	jnz     sig4            | if zero offset, try for zero or one base

	and     cx,cx           | zero argument ?
	jnz     sig1            | if zero then just give zero to kernel

	cmp	cx,*1           | special ignore argument?		* M000 *
	jz	sig1            | if one then just give one to kernel	* M000 *

sig4:
	.textref
	lea     cx,sigvec(bx)   | tell os the addr of our routine
				| NOTE: segment base not given
				|	code must be in first relative segment

|       (ax) = signal number
|       ((sp)) = old signal value

sig1:   xchg    ax,bx           | (bx) = signal number
	mov     ax,*.sigs
	calli   @Iscadr         | call XENIX
	pop     bx              | (bx) = original table entry offset
	pop	dx		| (dx) = base for original table entry
	jb      sig3            | if error
	testb   al,*1           | if ignored, just return
	jnz     sig5
	xchg    ax,bx           | (ax) = previous table entry, not os value
	j       sig5

sig2:
	mov     ax,*22.         | Invalid argument
sig3:
	mov     _errno,ax
	mov     ax,*-1
	mov	dx,*-1
sig5:
	reti                    | return


|**	sigtrp - Handle signal traps
|
|	When a signal is intercepted, the system calls enters the
|	user-specified signal routine as if an interrupt had occured.
|	The signal routine must do an 'iret' back to the user task's
|	interrupted execution path.
|
|	Naturally, 'c' does not know how to generate this, so we
|	intersperse this 'thunk', which gets control from unix, passes
|	it to the user's routine, then properly restores control
|	to the interrupted execution path.
|
|       ENTRY   ((sp)+6) = return flags
|               ((sp)+4) = return cs
|               ((sp)+2) = return ip
|               ((sp))   = sigvec entry + 3

sigtrp:
|       cli
|       push    ax
|       pop     ax
|       int     *3
	push    bp		| save all registers
	mov     bp,sp
	push    ax
	push    bx
	push    cx
	push    dx
	push    si
	push    di
	mov     ax,*2(bp)       | (ax) = return addr into sigvec
	sub     ax,#sigvec+3    | (ax) = signal number * 4
	mov	bx,ax		| (bx) = offset into sigtab	* M001 *
	shr     ax,*1
	shr     ax,*1           | /4
	push    ax              | pass signal # as param
	calli	@sigtab(bx)     | call 'c' routine		* M001 *
	pop     di              | remove argument
	pop     di
	pop     si
	pop     dx
	pop     cx
	pop     bx
	pop     ax
	pop     bp              | restore regs
	add     sp,*2           | skip 'sigvec' return addr
	iret


|**	sigvec - signal vector routine
|
|	unix is given an address in this table as the 'user routine'
|	to activate when a signal is received.  This 'routine' simply
|	encodes what the signal number was (via the return address)
|	and lets 'sigtrap' handle things.

sigvec:
	call    sigtrp			|signal 0
	nop				| each call is 4 bytes
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp
	nop
	call    sigtrp                 |through 16

	.data

|*	User signal vectors
|*
|*	these cells contain the address of the user's signal routine

sigtab: .word   0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0

	.end
