|       mcount - count subroutine calls during profiling
|
|       Each procedure, upon entry, calls here.  Each procedure has
|       a static cell; the first time through here a 6-byte count
|       block is allocated.  Its addr+2 is stored in the proc's static
|       cell so that subsequent calls here can just do the increment.
|
|       area+0: .word   proc            holds retadr to proc
|       area+2: .word   hi,lo           32 bit count field
|
|       The analyzer program uses the proc addr to associate the count
|       with a proc name.
|
|       NOTE that there is no check for overflow of the 'countbase'
|       area, if we get more procs than countbase was setup for, its
|       bad news.
|
|       ENTRY   (ax) = pointer to proc's private static word


.globl	mcount
.comm	countbase,2

mcount:
	xchg    ax,bx           | (bx) = per routine counter pointer
	mov     ax,(bx)
	and     ax,ax
	jnz     mco1            | already have a counter cell
	mov     ax,countbase
	and     ax,ax
	jz      mco2            | nowhere to count
	add     countbase,*6
	xchg    ax,bx           | (bx) = fwa of 6-byte proc counter
	pop     cx
	push    cx
	mov     (bx),cx         | store ret addr in 6-byte area
	add     bx,*2
	xchg    ax,bx
	mov     (bx),ax         | store counter addr for next time

|       Have a counter cell allocated.
|
|       (ax) = counter cell addr

mco1:
	xchg    ax,bx
	inc     *2(bx)
	jnz     mco2
	inc     (bx)
mco2:   reti
