|       MCRT0.S - C runtime startoff including monitoring
|
|       Adapted from pdp-11 version  (JGL, 05/12/82)
|
|       This routine, like crt0.s, gets initial control from the OS
|       and calls the user's "main()" routine.  This special version
|       also sets up special monitoring; procedure call counts and
|       ip-histogramming.
|
|       This routine allocates the buffer used to hold the info.
|       Its format:
|
|       b:      .blkb   6       | scaling info used by prof(1)
|               .blkb   6*n     | procedure call counts, see mcount.s
|               .blkb   2*m     | ip-histogram cells
|
|       The c compiler presumably generated code to call mcount.s, which
|       counts the procedure call frequency.  This routine sets up
|       'countbase', which mcount.s needs.
|
|       The monitor(3) routine (called below) tells the kernel where
|       to do the ip-histogramming.



cbufs   = 300.                  | can count 300 procedures
.exit   = 1
write	= 4

	.comm   _environ,2.


	.globl  _monitor
	.globl  _sbrk
	.globl  _main
	.globl  _exit
	.globl  _environ
	.globl  _etext
	.globl  __cleanup
	.globl  countbase


	.text

start:                          |******   Start executing here

|       These first 128 bytes will be set by the kernel when the
|       module is first executed.  The kernel assumes that the first
|       128 bytes of code space are available to it.  User code
|       must do long calls into one of 8 magic offsets in this
|       128-byte area:

	j       start0
	j       .               | system call           +2
	j       .               | stack growth          +4
	j       .               | stack adjust          +6
	j       .               | reserved              +8
	j       .               | reserved              +10
	j       .               | reserved              +12
	j       .               | reserved              +14
	j       .               | reserved              +16


	.=start+128             | kernal loads first 128 bytes


start0: mov     ax,cs           | temporary until new linker, see
	mov     Isgadr+2,ax     |       'Isgadr'...
	mov     Iscadr+2,ax
	mov     bx,sp           | bx->ac,avs,envs
	sub     sp,*4           | space for real arguments
	add     bx,*2
	push    bx              | main (argv,
	push    *-2(bx)         | main (argc, argv
s1:
	add     bx,*2           | next argv
	test    (bx),#/ffff     | look at pointer
	jnz     s1              | loop until zero argv

	add     bx,*2           | first env
	mov     si,sp
	mov     *4(si),bx       | main (argv, argv, envv)
	mov     _environ,bx     | place for people to look for them

|       setup monitoring buffer

	mov     ax,#cbufs
	push    ax              | monitor (   ,   ,      ,      ,nfunc)
	mov     ax,#_etext+7
	sub     ax,#myend       | (ax) = bytes of user text, rounded up 8
	shr     ax,*1
	shr     ax,*1
	shr     ax,*1           | figure one cell per 8 bytes...
	add     ax,#3*cbufs+3   | add space for 'cbuf' proc call counts
	push    ax              | monitor (   ,   ,      ,bufsize,nfunc)
	add     ax,ax
	push    ax
	calli   _sbrk           | ask for enough bytes
	add     sp,*2
	push    ax              | monitor (   ,   ,bufadr,bufsize,nfunc)
	inc     ax
	jz      nomem           | not enough for monitoring
	add     ax,*5           | (+6 with preceding inc)
	mov     countbase,ax    | store addr of 1st free proc count area
	mov     ax,#_etext
	push    ax              | monitor (   ,lwa,bufadr,bufsize,nfunc)
	mov     ax,#myend
	push    ax              | monitor (fwa,lwa,bufadr,bufsize,nfunc)
	calli   _monitor        | setup monitoring
	add     sp,*10.

	calli   _main
	push    ax              | pass return value to exit
crex:   calli   _exit           | call portable exit
	jmp     crex            | should never return from _exit


|       couldnt get monitoring memory

nomem:
	mov     ax,*memmsgl
	push    ax
	mov     ax,#memmsg
	push    ax
	mov     ax,*2
	push    ax
	calli   _write          | write (2, "No space for monitor buffer",n)
	mov     ax,*1
	push    ax              | use '1' for return code
	calli   _exit


|**     exit - identical to cuexit.s except for monitor call

_exit:
	calli   __cleanup
	sub     ax,ax
	push    ax
	calli   _monitor        | monitor(0) - write stuff to disk
	add     sp,*2
	mov     ax,*.exit
	jmp     sys1a           | exit system with _exit's argument

myend:


	.data
	.word   0       | loc 0 for I/D; null ptr points here.
memmsg: .ascii  #No space for monitor buffer\n#
memmsgl =       .-memmsg

|       Until we're using the new loader, its a pain generating segmented
|       addresses, and the library routines will call the Ios_?? entrys
|       indirectly through these addresses.  The segment values are filled
|       in when crt0 gets control

	.globl  Isgadr,Iscadr

Isgadr: .word   4               | stack growth
	.word   0
Iscadr: .word   2               | system call
	.word   0


	.bss
_environ:
	.=.+2
countbase:
	.=.+2
